import {
  ButtonProps,
  Tooltip,
  useDisclosure,
  Text,
  Modal,
  ModalOverlay,
  ModalContent,
  ModalHeader,
  ModalCloseButton,
  ModalBody,
  ModalFooter,
  HStack,
  Button,
  Select,
  Center,
  Icon,
  VStack,
  Badge,
  IconButton,
} from '@chakra-ui/react'
import React from 'react'
import { FileWithName, Implementation, Pattern } from 'src/models'
import { useRunTestsButton } from './use-run-tests-button'
import { FaPaperclip, FaTrash } from 'react-icons/fa'
import prettyBytes from 'pretty-bytes'
import { DropzoneState } from 'react-dropzone'
import { RunTestsStep1Mutation } from '../query-builder/run-tests-step-1-mutation'

const FORM_ID = 'run-tests-form'

export type RunTestsButtonProps = {
  implementation: Implementation
  pattern: Pattern
  hasTooltip?: boolean
  children: React.ReactElement<ButtonProps>
}

export const RunTestsButton: React.FC<RunTestsButtonProps> = ({
  implementation,
  pattern,
  hasTooltip = false,
  children,
}) => {
  const { isOpen, onOpen, onClose } = useDisclosure()

  const {
    formalizations,
    formalizationId,
    setFormalizationId,
    isValid,
    isSubmitting,
    strategy,
    onRemoveStrategy,
    strategyDropzoneState,
    testCasesInstantiation,
    testCasesInstantiationDropzoneState,
    onRemoveTestCasesInstantiation,
    handleOnRunTests,
  } = useRunTestsButton({
    patternId: pattern.id,
  })

  const btn = React.cloneElement(children, { onClick: onOpen })

  return (
    <>
      <Tooltip isDisabled={!hasTooltip} label={<Text>Correr tests</Text>}>
        {btn}
      </Tooltip>

      <Modal isOpen={isOpen} onClose={onClose}>
        <ModalOverlay />
        <ModalContent>
          <ModalHeader>
            <Text fontSize="md">
              Correr tests de la implementacion {implementation.name} del patron{' '}
              {pattern.name}
            </Text>
          </ModalHeader>
          <ModalCloseButton mt="-2" mr="-2" />
          <form id={FORM_ID}>
            <ModalBody>
              <HStack spacing="4">
                <Text>Formalizacion:</Text>
                <Select
                  name="formalizationId"
                  value={formalizationId}
                  onChange={(e) => setFormalizationId(e.target.value)}
                >
                  {formalizations.map((formalization, index) => {
                    return (
                      <option
                        key={`formalization-${index}`}
                        value={formalization.id}
                      >
                        {formalization.name}
                      </option>
                    )
                  })}
                </Select>
              </HStack>
              <FileSelector
                source="Strategy"
                fileWithName={strategy}
                dropzoneState={strategyDropzoneState}
                handleOnRemove={onRemoveStrategy}
              />

              <FileSelector
                source="Test cases instantiation"
                fileWithName={testCasesInstantiation}
                dropzoneState={testCasesInstantiationDropzoneState}
                handleOnRemove={onRemoveTestCasesInstantiation}
              />
            </ModalBody>
          </form>
          <ModalFooter>
            <HStack>
              <Button variant="outline" onClick={onClose}>
                Cancelar
              </Button>
              <Button
                form={FORM_ID}
                onClick={handleOnRunTests}
                variant="solid"
                colorScheme="blue"
                type="submit"
                isLoading={isSubmitting}
                isDisabled={!isValid}
              >
                Correr
              </Button>
            </HStack>
          </ModalFooter>
        </ModalContent>
      </Modal>
    </>
  )
}

type FileSelectorProps = {
  source: string
  fileWithName?: FileWithName
  dropzoneState: DropzoneState
  handleOnRemove: () => void
}
const FileSelector: React.FC<FileSelectorProps> = ({
  source,
  fileWithName,
  dropzoneState,
  handleOnRemove,
}) => {
  return (
    <VStack pt="4">
      {!fileWithName && (
        <div {...dropzoneState.getRootProps()}>
          <Center w="100%">
            <input aria-label="upload" {...dropzoneState.getInputProps()} />
            <HStack>
              <Button
                aria-label="add strategy"
                variant="unstyled"
                justifyContent="flex-start"
                iconSpacing="2"
                display="flex"
                fontWeight="400"
                rightIcon={<Icon as={FaPaperclip} />}
              >
                Agregar {source}
              </Button>
            </HStack>
          </Center>
        </div>
      )}

      <VStack w="full">
        {fileWithName && (
          <>
            <Badge>
              {source} {fileWithName.name}
            </Badge>
            <HStack
              spacing="0"
              w="full"
              border="1px"
              borderColor="gray.200"
              borderRadius="sm"
              bg="white"
              width="full"
              height="min-content"
              mt="0"
              minW="0"
              flex="1"
              alignItems="flex-start"
              fontSize="xs"
              h="52px"
            >
              <Center bg="blue.600" borderLeftRadius="base" px="4" h="52px">
                <Text
                  fontWeight="medium"
                  variant="-1"
                  color="white"
                  textTransform="uppercase"
                >
                  {fileWithName.file.name.split('.')[1] ?? ''}
                </Text>
              </Center>
              <HStack
                h="full"
                borderColor="gray.200"
                borderWidth="px"
                borderStyle="solid"
                borderRadius="base"
                borderLeft="none"
                borderLeftRadius="none"
                ml="0"
                w="full"
                minWidth="0"
                flex="1"
                pt="2"
              >
                <VStack
                  justifyContent={
                    fileWithName.file.size ? 'center' : 'flex-start'
                  }
                  spacing="1"
                  minW="0"
                  flex="1"
                >
                  <Text
                    variant="-1"
                    isTruncated={true}
                    w="full"
                    textAlign="center"
                  >
                    {fileWithName.file.name}
                  </Text>

                  <Text variant="-2" color="gray.500">
                    {prettyBytes(fileWithName.file.size ?? 0)}
                  </Text>
                </VStack>
                <IconButton
                  aria-label="remove formalization"
                  variant="unstyled"
                  color="blue.400"
                  onClick={handleOnRemove}
                  icon={<Icon as={FaTrash} />}
                />
              </HStack>
            </HStack>
          </>
        )}
      </VStack>
    </VStack>
  )
}
