import React from 'react'
import { useDropzone } from 'react-dropzone'
import { useQueryClient } from 'react-query'
import { FileWithName, Formalization, Pattern } from 'src/models'
import {
  MAX_FILE_SIZE,
  SIZE_TOO_LARGE,
} from 'src/pages/patterns/add-pattern/use-add-formalizations'
import { GetFormalizationsQuery } from 'src/pages/patterns/query-builder/get-formalizations-query'
import { RunTestsStep1Mutation } from '../query-builder/run-tests-step-1-mutation'

export function useRunTestsButton({ patternId }: { patternId: Pattern['id'] }) {
  const queryClient = useQueryClient()

  React.useEffect(() => {
    queryClient.invalidateQueries(GetFormalizationsQuery.key)
  }, [])

  // Formalization
  const [formalizationId, setFormalizationId] =
    React.useState<Formalization['id']>('')

  const formalizationsQuery = GetFormalizationsQuery.useQuery(
    {
      patternId: patternId,
    },
    {
      onSettled: (data) => {
        setFormalizationId(data[0]?.id ?? '')
      },
    }
  )

  // Strategy
  const [strategy, setStrategy] = React.useState<FileWithName>()
  const onDropStrategy = React.useCallback((acceptedFiles: File[]) => {
    if (acceptedFiles.length > 1) {
      throw Error('Expected 1 strategy file')
    }
    const res = acceptedFiles.map(
      (f) =>
        ({
          file: f,
          name: f.name.split('.')[0] ?? f.name,
        } as FileWithName)
    )
    setStrategy(res[0])

    return [...res]
  }, [])

  const strategyDropzoneState = useDropzone({
    onDrop: onDropStrategy,
    noDrag: true,
    validator: (file) => {
      if (file.size > MAX_FILE_SIZE) {
        return {
          code: SIZE_TOO_LARGE,
          message: `Maximo ${MAX_FILE_SIZE} MB.`,
        }
      }
      return null
    },
  })

  const onRemoveStrategy = () => setStrategy(null)

  // Test cases instantiation
  const [testCasesInstantiation, setTestCasesInstantiation] =
    React.useState<FileWithName>()
  const onDropTestCasesInstantiation = React.useCallback(
    (acceptedFiles: File[]) => {
      if (acceptedFiles.length > 1) {
        throw Error('Expected 1 test file')
      }
      const res = acceptedFiles.map(
        (f) =>
          ({
            file: f,
            name: f.name.split('.')[0] ?? f.name,
          } as FileWithName)
      )
      setTestCasesInstantiation(res[0])

      return [...res]
    },
    []
  )

  const testCasesInstantiationDropzoneState = useDropzone({
    onDrop: onDropTestCasesInstantiation,
    noDrag: true,
    validator: (file) => {
      if (file.size > MAX_FILE_SIZE) {
        return {
          code: SIZE_TOO_LARGE,
          message: `Maximo ${MAX_FILE_SIZE} MB.`,
        }
      }
      return null
    },
  })

  const onRemoveTestCasesInstantiation = () => setTestCasesInstantiation(null)

  // Utils

  const isValid = React.useMemo(() => {
    return (
      Boolean(formalizationId) &&
      Boolean(strategy) &&
      Boolean(testCasesInstantiation)
    )
  }, [formalizationId, strategy, testCasesInstantiation])

  const [isSubmitting, setIsSubmitting] = React.useState(false)

  const runTestsMutation = RunTestsStep1Mutation.useMutation({
    onSuccess: (result) => console.log(result),
  })

  const handleOnRunTests = async () => {
    if (!isValid) return

    setIsSubmitting(true)

    await runTestsMutation.mutateAsync({
      id: formalizationId,
      strategyFile: strategy.file,
      testCasesInstantiationFile: testCasesInstantiation.file,
    })

    setIsSubmitting(false)
  }

  return {
    formalizations: formalizationsQuery.data ?? [],
    formalizationId,
    setFormalizationId,
    isValid,
    isSubmitting,
    strategyDropzoneState,
    strategy,
    onRemoveStrategy,
    testCasesInstantiationDropzoneState,
    testCasesInstantiation,
    onRemoveTestCasesInstantiation,
    handleOnRunTests,
  }
}
